@@ -62,8 +62,10 @@ panel.qqmath <-
 qqmath <-
     function(formula,
              data = parent.frame(),
+             allow.multiple = FALSE,
+             outer = TRUE,
+             auto.key = FALSE,
              aspect = "fill",
-             layout = NULL,
              panel = "panel.qqmath",
              prepanel = NULL,
              scales = list(),
@@ -73,8 +75,10 @@ qqmath <-
              xlim,
              ylab,
              ylim,
-             f.value = ppoints,
+###          f.value = ppoints,
+             f.value = NULL,
              distribution = qnorm,
+             drop.unused.levels = TRUE,
              ...,
              subscripts = !is.null(groups),
              subset = TRUE)
@@ -83,11 +87,27 @@ qqmath <-
     ## dots <- eval(substitute(list(...)), data, parent.frame())
     dots <- list(...)
 
+    groups <- eval(substitute(groups), data, parent.frame())
+    subset <- eval(substitute(subset), data, parent.frame())
+
+    ## Step 1: Evaluate x, y, etc. and do some preprocessing
+
+    form <-
+        latticeParseFormula(formula, data, subset = subset,
+                            groups = groups, multiple = allow.multiple,
+                            outer = outer, subscripts = TRUE,
+                            drop = drop.unused.levels)
+
+    groups <- form$groups
+
     if (!is.function(panel)) panel <- eval(panel)
     if (!is.function(strip)) strip <- eval(strip)
+    
+    if ("subscripts" %in% names(formals(panel))) subscripts <- TRUE
+    if (subscripts) subscr <- form$subscr
 
     prepanel <-
-        if (is.function(prepanel)) prepanel 
+        if (is.function(prepanel)) prepanel
         else if (is.character(prepanel)) get(prepanel)
         else eval(prepanel)
 
@@ -97,25 +117,21 @@ qqmath <-
         else if (is.character(distribution)) get(distribution)
         else eval(distribution)
 
-    ## Step 1: Evaluate x, y, etc. and do some preprocessing
-    
-    form <- latticeParseFormula(formula, data)
     cond <- form$condition
     number.of.cond <- length(cond)
     x <- form$right
+
     if (number.of.cond == 0) {
         strip <- FALSE
         cond <- list(as.factor(rep(1, length(x))))
-        layout <- c(1,1,1)
         number.of.cond <- 1
     }
 
-    groups <- eval(substitute(groups), data, parent.frame())
-    subset <- eval(substitute(subset), data, parent.frame())
-    if ("subscripts" %in% names(formals(eval(panel)))) subscripts <- TRUE
-    if(subscripts) subscr <- seq(along=x)
-    x <- x[subset, drop = TRUE]
-    if (subscripts) subscr <- subscr[subset, drop = TRUE]
+
+
+    
+
+
 
     if(missing(ylab)) ylab <- form$right.name
     if(missing(xlab)) xlab <- distribution.name
@@ -126,23 +142,18 @@ qqmath <-
     ## less complicated components:
 
     foo <- do.call("trellis.skeleton",
-                   c(list(aspect = aspect,
+                   c(list(cond = cond,
+                          aspect = aspect,
                           strip = strip,
                           panel = panel,
                           xlab = xlab,
-                          ylab = ylab), dots))
+                          ylab = ylab,
+                          xlab.default = deparse(substitute(distribution)),
+                          ylab.default = form$right.name), dots))
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
     foo$call <- match.call()
-    foo$fontsize.normal <- 10
-    foo$fontsize.small <- 8
-
-    ## This is for cases like xlab/ylab = list(cex=2)
-    if (is.list(foo$xlab) && !is.characterOrExpression(foo$xlab$label))
-        foo$xlab$label <- deparse(substitute(distribution))
-    if (is.list(foo$ylab) && !is.characterOrExpression(foo$ylab$label))
-        foo$ylab$label <- form$right.name
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -186,7 +197,6 @@ qqmath <-
     
     ## Step 5: Process cond
 
-    cond <- lapply(cond, as.factorOrShingle, subset, drop = TRUE)
     cond.max.level <- unlist(lapply(cond, nlevels))
 
 
@@ -196,77 +206,94 @@ qqmath <-
     if (!any(!id.na)) stop("nothing to draw")
     ## Nothing simpler ?
 
-    foo$condlevels <- lapply(cond, levels)
 
     ## Step 6: Evaluate layout, panel.args.common and panel.args
 
     foo$panel.args.common <- dots
     if (subscripts) foo$panel.args.common$groups <- groups
 
-    layout <- compute.layout(layout, cond.max.level, skip = foo$skip)
-    plots.per.page <- max(layout[1] * layout[2], layout[2])
-    number.of.pages <- layout[3]
-    foo$skip <- rep(foo$skip, length = plots.per.page)
-    foo$layout <- layout
-    nplots <- plots.per.page * number.of.pages
+    nplots <- prod(cond.max.level)
+    if (nplots != prod(sapply(foo$condlevels, length))) stop("mismatch")
+    foo$panel.args <- vector(mode = "list", length = nplots)
 
-    foo$panel.args <- as.list(1:nplots)
-    cond.current.level <- rep(1,number.of.cond)
-    panel.number <- 1 # this is a counter for panel number
-    for (page.number in 1:number.of.pages)
-        if (!any(cond.max.level-cond.current.level<0))
-            for (plot in 1:plots.per.page) {
 
-                if (foo$skip[plot]) foo$panel.args[[panel.number]] <- FALSE
-                else if(!any(cond.max.level-cond.current.level<0)) {
+    cond.current.level <- rep(1, number.of.cond)
 
-                    id <- !id.na
-                    for(i in 1:number.of.cond)
-                    {
-                        var <- cond[[i]]
-                        id <- id &
-                        if (is.shingle(var))
-                            ((var >=
-                              levels(var)[[cond.current.level[i]]][1])
-                             & (var <=
-                                levels(var)[[cond.current.level[i]]][2]))
-                        else (as.numeric(var) == cond.current.level[i])
-                    }
-                    if (any(id)) {
-                        foo$panel.args[[panel.number]] <-
-                            list(x = distribution(f.value(length(x[id]))), 
-                                 y = quantile(x[id], f.value(length(x[id]))))
-                    }
-                    else
-                        foo$panel.args[[panel.number]] <-
-                            list(x = numeric(0), y = numeric(0))
 
-                    if (subscripts)
-                        foo$panel.args[[panel.number]]$subscripts <-
-                            subscr[id]
+    for (panel.number in seq(length = nplots))
+    {
 
-                    cond.current.level <-
-                        cupdate(cond.current.level,
-                                cond.max.level)
-                }
+        id <- !id.na
+        for(i in 1:number.of.cond)
+        {
+            var <- cond[[i]]
+            id <- id &
+            if (is.shingle(var))
+                ((var >=
+                  levels(var)[[cond.current.level[i]]][1])
+                 & (var <=
+                    levels(var)[[cond.current.level[i]]][2]))
+            else (as.numeric(var) == cond.current.level[i])
+        }
+        if (any(id)) {
+            foo$panel.args[[panel.number]] <-
+                if (is.null(f.value)) # exact data instead of quantiles
+                    list(x = distribution(ppoints(length(x[id]))), 
+                         y = sort(x[id]))
+                else
+                    list(x = distribution(f.value(length(x[id]))), 
+                         y = quantile(x[id], f.value(length(x[id]))))
+        }
+        else
+            foo$panel.args[[panel.number]] <-
+                list(x = numeric(0), y = numeric(0))
 
-                panel.number <- panel.number + 1
-            }
+        if (subscripts)
+            foo$panel.args[[panel.number]]$subscripts <-
+                subscr[id]
 
-    foo <- c(foo,
-             limits.and.aspect(prepanel.default.qqmath,
-                               prepanel = prepanel, 
-                               have.xlim = have.xlim, xlim = xlim, 
-                               have.ylim = have.ylim, ylim = ylim, 
-                               x.relation = foo$x.scales$relation,
-                               y.relation = foo$y.scales$relation,
-                               panel.args.common = foo$panel.args.common,
-                               panel.args = foo$panel.args,
-                               aspect = aspect,
-                               nplots = nplots,
-                               x.axs = foo$x.scales$axs,
-                               y.axs = foo$y.scales$axs,
-                               distribution = distribution))
+        cond.current.level <-
+            cupdate(cond.current.level,
+                    cond.max.level)
+    }
+
+
+    more.comp <- c(limits.and.aspect(prepanel.default.qqmath,
+                                     prepanel = prepanel, 
+                                     have.xlim = have.xlim, xlim = xlim, 
+                                     have.ylim = have.ylim, ylim = ylim, 
+                                     x.relation = foo$x.scales$relation,
+                                     y.relation = foo$y.scales$relation,
+                                     panel.args.common = foo$panel.args.common,
+                                     panel.args = foo$panel.args,
+                                     aspect = aspect,
+                                     nplots = nplots,
+                                     x.axs = foo$x.scales$axs,
+                                     y.axs = foo$y.scales$axs,
+                                     distribution = distribution),
+                   cond.orders(foo))
+    foo[names(more.comp)] <- more.comp
+
+    if (is.null(foo$legend) && !is.null(groups) &&
+        (is.list(auto.key) || (is.logical(auto.key) && auto.key)))
+    {
+        foo$legend <-
+            list(list(fun = "drawSimpleKey",
+                      args =
+                      c(list(levels(as.factor(groups))),
+                        if (is.list(auto.key)) auto.key else list())))
+        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
+        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
+        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
+
+        names(foo$legend) <- 
+            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
+                "inside"
+            else
+                "top"
+        if (!is.null(foo$legend[[1]]$args$space))
+            names(foo$legend) <- foo$legend[[1]]$args$space
+    }
 
     class(foo) <- "trellis"
     foo
